SorhaQuestLog = LibStub("AceAddon-3.0"):NewAddon("SorhaQuestLog", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SorhaQuestLog")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local LSM = LibStub("LibSharedMedia-3.0") 
local db
local dbChar

LSM:Register("font", "framd", [[Interface\AddOns\SorhaQuestLog\Fonts\framd.ttf]])

BINDING_HEADER_SORHAQUESTLOG = "Sorha Quest Log";
BINDING_NAME_TOGGLE_SORHAQUESTLOG = L["Toggle Minion"]

--Frames
local fraSQLMainAnchor = nil
local dicButtons = {}
local intNumberOfFrames = 50
local strButtonPrefix = "SorhaQuestLogButton"

--Colour strings
local strHeaderColour = "|cff00aaff"
local strQuestTitleColour = "|cff00aaff"
local strObjectiveColour = "|cffffffff"
local strObjective00PlusColour = "|cffffffff"
local strObjective25PlusColour = "|cffffffff"
local strObjective50PlusColour = "|cffffffff"
local strObjective75PlusColour = "|cffffffff"
local strDoneObjectiveColour = "|cff00ffff"
local strObjectiveDescriptionColour = "|cff888888"

--Just some booleans etc
local blnInitialized = false
local blnIsUpdating = false
local blnWasAClick = false
local blnFirstQuestLogUpdate = true
local blnIgnoreUpdateEvents = false
local strZone = ""
local strSubZone = ""

local dicOutlines = {
	[""] = L["None"],
	["OUTLINE"] = L["Outline"],
	["THICKOUTLINE"] = L["Thick Outline"],
}

local defaults = { --Defaults Table
	profile = {
		Frames = {
			MainWindowLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			MainWindowScale = 1,
			MainWindowLocked = false,			
			QuestFontSize = 11,
			QuestFont = "framd",
			QuestFontOutline = "",
			HeaderFontSize = 11,
			HeaderFont = "framd",
			HeaderFontOutline = "",
			MinionWidth = 220,
			HeaderColour = {r = 0, g = 0.6, b = 1, a = 1},
			QuestTitleColour = {r = 1, g = 1, b = 1, a = 1},
			ObjectiveColour = {r = 1, g = 1, b = 1, a = 1},
			Objective00PlusColour = {r = 1, g = 0, b = 0, a = 1},
			Objective25PlusColour = {r = 1, g = 0.3, b = 0, a = 1},
			Objective50PlusColour  = {r = 1, g = 0.6, b = 0, a = 1},
			Objective75PlusColour = {r = 1, g = 1, b = 0, a = 1},
			ObjectiveDoneColour = {r = 0, g = 1, b = 1, a = 1},
			ObjectiveDescColour = {r = 0.5, g = 0.5, b = 0.5, a = 0.5},
			ShowItemButtons = true,
			HideCompletedObjectives = false,
			ShowNumberOfQuests = false,
			MoveItemsAndTooltipsRight = false,
			ShowDescWhenNoObjectives = false,
			HideSorhaQuestLog = false,
			CollapseToLeft = false,
			ShowAllQuests = false,
			AllowHiddenQuests = false,
			CollapseOnLeave = false,
			ExpandOnEnter = false,
			UseProgressiveColouring = false,
			UseQuestLevelColouring = true,
		},
	},
	char = {
		Quests = {
			HiddenQuests = {},
		},
	},
}

local function getOptions() --Creates Options table.
	if not SorhaQuestLog.options then
		SorhaQuestLog.options = {
			type = "group",
			name = "SorhaQuestLog",
			handler = SorhaQuestLog,
			args = {
				General = {
					name = L["General Options"],
					type = "group",
					order = 1,
					args = {
						MainHide = {
							name = L["Hide Minion"],
							desc = L["Hides/Shows the Main Frame"],
							type = "toggle",
							get = function() return db.Frames.HideSorhaQuestLog end,
							set = function()
										SorhaQuestLog:ToggleSorhaQuestLog()
									end,
							order = 1,
						},
						MainLock = {
							name = L["Lock Frame"],
							desc = L["Unlocks/Locks the Main Frame"],
							type = "toggle",
							get = function() return db.Frames.MainWindowLocked end,
							set = function()
										db.Frames.MainWindowLocked = not db.Frames.MainWindowLocked
										SorhaQuestLog:doAnchorBackgroundUpdate()
									end,
							order = 2,
						},
						HeaderMainFrameSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 10,
						},
						HeaderMainFrame = {
							name = L["Main Frame Settings"],
							type = "header",
							order = 11,
						},
						MainWidth = {
							order = 12,
							name = L["Width"],
							desc = L["Controls the width of the main window."],
							type = "range",
							min = 150, max = 300, step = 1,
							isPercent = false,
							get = function() return db.Frames.MinionWidth end,
							set = function(info, value)
								db.Frames.MinionWidth = value
								fraSQLMainAnchor:SetWidth(db.Frames.MinionWidth)
								SorhaQuestLog:UpdateMinion()
							end,
						},	
						Reset = {
							order = 13,
							type = "execute",
							name = L["Reset Main Frame"],
							desc = L["Resets Main Frame position"],
							func = function()
								db.Frames.MainWindowLocation.Point = "CENTER"
								db.Frames.MainWindowLocation.RelativePoint =  "CENTER"
								db.Frames.MainWindowLocation.X = 0
								db.Frames.MainWindowLocation.Y = 0
								fraSQLMainAnchor:StopMovingOrSizing()		
								fraSQLMainAnchor:ClearAllPoints()
								fraSQLMainAnchor:SetPoint(db.Frames.MainWindowLocation.Point, UIParent, db.Frames.MainWindowLocation.RelativePoint, db.Frames.MainWindowLocation.X, db.Frames.MainWindowLocation.Y)
							end,
						},
						HeaderMiscSettingsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 30,
						},
						HeaderMiscSettings = {
							name = L["Misc. Settings"],
							type = "header",
							order = 31,
						},
						ShowNumberQuestsToggle = {
							name = L["Show number of quests"],
							desc = L["Shows/Hides the number of quests"],
							type = "toggle",
							get = function() return db.Frames.ShowNumberOfQuests end,
							set = function()
								db.Frames.ShowNumberOfQuests = not db.Frames.ShowNumberOfQuests
								SorhaQuestLog:UpdateMinion()
							end,
							order = 32,
						},
						CollapseToLeftToggle = {
							name = L["Autoshrink to left"],
							desc = L["Shrinks the width down when the length of current quests is less then the max width\nNote: Doesn't work well with quests that wordwrap"],
							type = "toggle",
							get = function() return db.Frames.CollapseToLeft end,
							set = function()
								db.Frames.CollapseToLeft = not db.Frames.CollapseToLeft
								SorhaQuestLog:UpdateMinion()
							end,
							order = 33,
						},
						ItemsAndTooltipsRightToggle = {
							name = L["Display items and tooltips on right"],
							desc = L["Moves items and tooltips to the right"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.MoveItemsAndTooltipsRight end,
							set = function()
								db.Frames.MoveItemsAndTooltipsRight = not db.Frames.MoveItemsAndTooltipsRight
								SorhaQuestLog:UpdateMinion()
							end,
							order = 34,
						},
					},
				},
				QuestOptions = {
					name = L["Quest Options"],
					type = "group",
					order = 1,
					args = {
						DisplaySettingsHeader = {
							name = L["Display Settings"],
							type = "header",
							order = 1,
						},
						ShowItemButtonsToggle = {
							name = L["Show quest item buttons"],
							desc = L["Shows/Hides the quest item buttons"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.ShowItemButtons end,
							set = function()
								db.Frames.ShowItemButtons = not db.Frames.ShowItemButtons
								SorhaQuestLog:UpdateMinion()
							end,
							order = 2,
						},
						AllowHiddenQuestsToggle = {
							name = L["Allow quests to be hidden"],
							desc = L["Allows quests to be hidden and enables the show/hide button"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.AllowHiddenQuests end,
							set = function()
								db.Frames.AllowHiddenQuests = not db.Frames.AllowHiddenQuests
								SorhaQuestLog:doHiddenQuestsUpdate()
							end,
							order = 3,
						},	
						HeaderZoneHeadersSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 20,
						},
						HeaderZoneHeaders = {
							name = L["Zone Header Settings"],
							type = "header",
							order = 21,
						},
						ExpandOnEnterToggle = {
							name = L["Auto expand zones on enter"],
							desc = L["Automatically expands zone headers when you enter the zone"],
							type = "toggle",
							get = function() return db.Frames.ExpandOnEnter end,
							set = function()
								db.Frames.ExpandOnEnter = not db.Frames.ExpandOnEnter
								SorhaQuestLog:doHandleZoneChange()
							end,
							order = 22,
						},	
						CollapseOnLeaveToggle = {
							name = L["Auto collapse zones on exit"],
							desc = L["Automatically collapses zone headers when you exit the zone"],
							type = "toggle",
							get = function() return db.Frames.CollapseOnLeave end,
							set = function()
								db.Frames.CollapseOnLeave = not db.Frames.CollapseOnLeave
								SorhaQuestLog:doHandleZoneChange()
							end,
							order = 23,
						},	
						HeaderFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 24,
							name = L["Header Font"],
							desc = L["The font that the zone headers will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Frames.HeaderFont end,
							set = function(info, value)
								db.Frames.HeaderFont = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						HeaderFontOutlineSelect = {
							name = L["Header Font Outline"],
							desc = L["The outline that the zone headers will use"],
							type = "select",
							order = 25,
							values = dicOutlines,
							get = function() return db.Frames.HeaderFontOutline end,
							set = function(info, value)
								db.Frames.HeaderFontOutline = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						HeaderFontSize = {
							order = 26,
							name = L["Header Font Size"],
							desc = L["Controls the font size of the header label."],
							type = "range",
							min = 10, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Frames.HeaderFontSize end,
							set = function(info, value)
								db.Frames.HeaderFontSize = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						HeaderQuestsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 40,
						},
						HeaderQuests = {
							name = L["Quest Settings"],
							type = "header",
							order = 41,
						},
						HideCompletedObjectivesToggle = {
							name = L["Hide completed objectives"],
							desc = L["Shows/Hides completed objectives"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.HideCompletedObjectives end,
							set = function()
								db.Frames.HideCompletedObjectives = not db.Frames.HideCompletedObjectives
								SorhaQuestLog:UpdateMinion()
							end,
							order = 42,
						},
						ShowDescWhenNoObjectivesToggle = {
							name = L["Display quest description if not objectives"],
							desc = L["Displays a quests description if there are no objectives available"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.ShowDescWhenNoObjectives end,
							set = function()
								db.Frames.ShowDescWhenNoObjectives = not db.Frames.ShowDescWhenNoObjectives
								SorhaQuestLog:UpdateMinion()
							end,
							order = 43,
						},	
						QuestFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 44,
							name = L["Quest Font"],
							desc = L["The font that the quest titles will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Frames.QuestFont end,
							set = function(info, value)
								db.Frames.QuestFont = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						QuestFontOutlineSelect = {
							name = L["Quest Font Outline"],
							desc = L["The outline that the quest titles will use"],
							type = "select",
							order = 45,
							values = dicOutlines,
							get = function() return db.Frames.QuestFontOutline end,
							set = function(info, value)
								db.Frames.QuestFontOutline = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
						QuestFontSize = {
							order = 46,
							name = L["Quest Font Size"],
							desc = L["Controls the font size of the quest info."],
							type = "range",
							min = 10, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Frames.QuestFontSize end,
							set = function(info, value)
								db.Frames.QuestFontSize = value
								SorhaQuestLog:UpdateMinion()
							end,
						},
					},
				},
				ColourOptions = {
					name = L["Colour Options"],
					type = "group",
					order = 2,
					args = {
						HeaderColourSettings = {
							name = L["Colour Settings"],
							type = "header",
							order = 1,
						},
						UseQuestLevelColouringToggle = {
							name = L["Use quest level colouring"],
							desc = L["Uses the level of the quest relative to you to colour the quest title"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.UseQuestLevelColouring end,
							set = function()
								db.Frames.UseQuestLevelColouring = not db.Frames.UseQuestLevelColouring
								SorhaQuestLog:UpdateMinion()
							end,
							order = 2,
						},
						UseProgressiveColouringToggle = {
							name = L["Use gradual objective colouring"],
							desc = L["Colours quest objectives based on completion percentage"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.UseProgressiveColouring end,
							set = function()
								db.Frames.UseProgressiveColouring = not db.Frames.UseProgressiveColouring
								SorhaQuestLog:UpdateMinion()
							end,
							order = 3,
						},						
						HeaderMainColoursSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 9,
						},
						HeaderMainColours = {
							name = L["Main Colours"],
							type = "header",
							order = 10,
						},						
						HeaderColour = {
							name = L["Zone Header Colour"],
							desc = L["Sets the color for the header of each zone"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Frames.HeaderColour.r, db.Frames.HeaderColour.g, db.Frames.HeaderColour.b, db.Frames.HeaderColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.HeaderColour.r = r
									db.Frames.HeaderColour.g = g
									db.Frames.HeaderColour.b = b
									db.Frames.HeaderColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 11,
						},
						QuestTitleColour = {
							name = L["Quest titles"],
							desc = L["Sets the color for the quest titles if colouring by level is off"],
							type = "color",
							disabled = function() return db.Frames.UseQuestLevelColouring end,
							hasAlpha = true,
							get = function() return db.Frames.QuestTitleColour.r, db.Frames.QuestTitleColour.g, db.Frames.QuestTitleColour.b, db.Frames.QuestTitleColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.QuestTitleColour.r = r
									db.Frames.QuestTitleColour.g = g
									db.Frames.QuestTitleColour.b = b
									db.Frames.QuestTitleColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 12,
						},
						ObjectiveColour = {
							name = L["Incomplete objective colour"],
							desc = L["Sets the color for the incomplete objectives"],
							type = "color",
							width = "full",
							hasAlpha = true,
							get = function() return db.Frames.ObjectiveColour.r, db.Frames.ObjectiveColour.g, db.Frames.ObjectiveColour.b, db.Frames.ObjectiveColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.ObjectiveColour.r = r
									db.Frames.ObjectiveColour.g = g
									db.Frames.ObjectiveColour.b = b
									db.Frames.ObjectiveColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 13,
						},
						DoneObjectiveColour = {
							name = L["Complete objective colour"],
							desc = L["Sets the color for the complete objectives"],
							type = "color",
							width = "full",
							hasAlpha = true,
							get = function() return db.Frames.ObjectiveDoneColour.r, db.Frames.ObjectiveDoneColour.g, db.Frames.ObjectiveDoneColour.b, db.Frames.ObjectiveDoneColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.ObjectiveDoneColour.r = r
									db.Frames.ObjectiveDoneColour.g = g
									db.Frames.ObjectiveDoneColour.b = b
									db.Frames.ObjectiveDoneColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 14,
						},
						NoObjectivesColour = {
							name = L["No objectives description colour"],
							desc = L["Sets the color for the description displayed when there is no quest objectives"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Frames.ObjectiveDescColour.r, db.Frames.ObjectiveDescColour.g, db.Frames.ObjectiveDescColour.b, db.Frames.ObjectiveDescColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.ObjectiveDescColour.r = r
									db.Frames.ObjectiveDescColour.g = g
									db.Frames.ObjectiveDescColour.b = b
									db.Frames.ObjectiveDescColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 15,
						},
						HeaderGradualColoursSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 30,
						},
						HeaderGradualColours = {
							name = L["Gradual objective Colours"],
							type = "header",
							order = 31,
						},	
						Objective00PlusColour = {
							name = L["0% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 0% complete"],
							type = "color",
							disabled = function() return not db.Frames.UseProgressiveColouring end,
							hasAlpha = true,
							get = function() return db.Frames.Objective00PlusColour.r, db.Frames.Objective00PlusColour.g, db.Frames.Objective00PlusColour.b, db.Frames.Objective00PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.Objective00PlusColour.r = r
									db.Frames.Objective00PlusColour.g = g
									db.Frames.Objective00PlusColour.b = b
									db.Frames.Objective00PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 32,
						},
						Objective25PlusColour = {
							name = L["25% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 25% complete"],
							type = "color",
							disabled = function() return not db.Frames.UseProgressiveColouring end,
							hasAlpha = true,
							get = function() return db.Frames.Objective25PlusColour.r, db.Frames.Objective25PlusColour.g, db.Frames.Objective25PlusColour.b, db.Frames.Objective25PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.Objective25PlusColour.r = r
									db.Frames.Objective25PlusColour.g = g
									db.Frames.Objective25PlusColour.b = b
									db.Frames.Objective25PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 33,
						},
						Objective50PlusColour = {
							name = L["50% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 50% complete"],
							type = "color",
							disabled = function() return not db.Frames.UseProgressiveColouring end,
							hasAlpha = false,
							get = function() return db.Frames.Objective50PlusColour.r, db.Frames.Objective50PlusColour.g, db.Frames.Objective50PlusColour.b, db.Frames.Objective50PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.Objective50PlusColour.r = r
									db.Frames.Objective50PlusColour.g = g
									db.Frames.Objective50PlusColour.b = b
									db.Frames.Objective50PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 34,
						},
						Objective75PlusColour = {
							name = L["75% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 75% complete"],
							type = "color",
							disabled = function() return not db.Frames.UseProgressiveColouring end,
							hasAlpha = true,
							get = function() return db.Frames.Objective75PlusColour.r, db.Frames.Objective75PlusColour.g, db.Frames.Objective75PlusColour.b, db.Frames.Objective75PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Frames.Objective75PlusColour.r = r
									db.Frames.Objective75PlusColour.g = g
									db.Frames.Objective75PlusColour.b = b
									db.Frames.Objective75PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 35,
						},
						
						
					},
				},
			},
		}
		SorhaQuestLog.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(SorhaQuestLog.db)
		SorhaQuestLog.options.args.profiles.order = 9
	end
	return SorhaQuestLog.options
	
end

function SorhaQuestLog:OnInitialize() --Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("SorhaQuestLogDB", defaults, true)
	db = self.db.profile
	dbChar = self.db.char
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
	self:SetupOptions()
	self:SetupColours()
end

function SorhaQuestLog:OnEnable() --Called when the addon is enabled
	strZone = GetRealZoneText()
	strSubZone = GetSubZoneText()
	self:CreateLayout()
	self:RegisterEvent("QUEST_LOG_UPDATE")	
	self:RegisterEvent('PLAYER_LEVEL_UP');
	self:RegisterEvent("ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_INDOORS")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	
	blnInitialized = true
end

function SorhaQuestLog:QUEST_LOG_UPDATE(...)
	if (db.Frames.HideSorhaQuestLog == false) then
		if (blnWasAClick == true) then -- Was the event called from a click (Skips the delay on header collapses etc)
			blnIsUpdating = true
			self:UpdateMinion()
		elseif (blnIsUpdating == false) then --If not updating then update, forces a 0.5 second delay between system called updates
			if (blnIgnoreUpdateEvents == false) then
				blnIsUpdating = true
				self:ScheduleTimer("UpdateMinion", 0.5)
			end
		end
	end
	if (blnFirstQuestLogUpdate == true) then
		blnFirstQuestLogUpdate = false
		self:CleanupHiddenQuestList()
	end
end

function SorhaQuestLog:PLAYER_LEVEL_UP(...)
	if (db.Frames.HideSorhaQuestLog == false) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:ZONE_CHANGED(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:ZONE_CHANGED_INDOORS(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:ZONE_CHANGED_NEW_AREA(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:RefreshConfig()
	db = self.db.profile
	fraSQLMainAnchor:SetWidth(db.Frames.MinionWidth)
	self:doColourChange()
	self:doAnchorBackgroundUpdate()
	self:UpdateMinion()
end

function SorhaQuestLog:CreateLayout()
	fraSQLMainAnchor = self:doCreateFrame("FRAME","SorhaQuestLogMainAnchor",UIParent,db.Frames.MinionWidth,40,1,"BACKGROUND",1, db.Frames.MainWindowLocation.Point, UIParent, db.Frames.MainWindowLocation.RelativePoint, db.Frames.MainWindowLocation.X, db.Frames.MainWindowLocation.Y, 1)
	fraSQLMainAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraSQLMainAnchor:SetMovable(true)
	fraSQLMainAnchor:SetClampedToScreen(true)
	fraSQLMainAnchor:RegisterForDrag("LeftButton")
	fraSQLMainAnchor:SetScript("OnDragStart", fraSQLMainAnchor.StartMoving)
	fraSQLMainAnchor:SetScript("OnDragStop",  function(self)
		fraSQLMainAnchor:StopMovingOrSizing()
		strPoint, tempB, strRelativePoint, intPosX, intPosY = fraSQLMainAnchor:GetPoint()
		db.Frames.MainWindowLocation.Point = strPoint
		db.Frames.MainWindowLocation.RelativePoint = strRelativePoint
		db.Frames.MainWindowLocation.X = intPosX
		db.Frames.MainWindowLocation.Y = intPosY
	end)
	fraSQLMainAnchor.objFontString = fraSQLMainAnchor:CreateFontString(nil, "OVERLAY");
	fraSQLMainAnchor.objFontString:SetFont(LSM:Fetch("font", db.Frames.HeaderFont), 14, db.Frames.HeaderFontOutline)
	
	fraSQLMainAnchor.objFontString:SetJustifyH("LEFT")
	fraSQLMainAnchor.objFontString:SetJustifyV("TOP")
	fraSQLMainAnchor.objFontString:SetText("");
	fraSQLMainAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	fraSQLMainAnchor.objFontString:SetShadowOffset(1, -1)
	
	fraSQLMainAnchor.buttonShowHidden = self:doCreateLooseFrame("BUTTON","SQLShowHiddenButton",fraSQLMainAnchor,16,16,1,"LOW",1,1)
	fraSQLMainAnchor.buttonShowHidden:SetPoint("TOPLEFT", fraSQLMainAnchor, "TOPLEFT", 0, -6);
	fraSQLMainAnchor.buttonShowHidden:SetBackdrop({bgFile="Interface\\AddOns\\SorhaQuestLog\\Textures\\button1.tga", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=0, tileSize=0, edgeSize=1, insets={left=-0, right=0, top=0, bottom=0}})
	fraSQLMainAnchor.buttonShowHidden:SetBackdropBorderColor(0, 0, 0, 0)
	
	
	fraSQLMainAnchor.buttonShowHiddenHover = self:doCreateLooseFrame("FRAME","SQLShowHiddenButtonHover",fraSQLMainAnchor.buttonShowHidden,fraSQLMainAnchor.buttonShowHidden:GetWidth(),fraSQLMainAnchor.buttonShowHidden:GetHeight(),1,"LOW",1,1)
	fraSQLMainAnchor.buttonShowHiddenHover:SetPoint("TOPLEFT", fraSQLMainAnchor.buttonShowHidden, "TOPLEFT", 0, 0);
	fraSQLMainAnchor.buttonShowHiddenHover:SetBackdrop({bgFile="Interface\\AddOns\\SorhaQuestLog\\Textures\\button1_hover.tga", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=0, tileSize=0, edgeSize=1, insets={left=-0, right=0, top=0, bottom=0}})
	fraSQLMainAnchor.buttonShowHiddenHover:SetBackdropColor(0, 0, 1, 1)
	fraSQLMainAnchor.buttonShowHiddenHover:SetBackdropBorderColor(0,0,0,0)
	fraSQLMainAnchor.buttonShowHiddenHover:SetAlpha(0)
	
	
	fraSQLMainAnchor.buttonShowHidden:RegisterForClicks("AnyUp")
	fraSQLMainAnchor.buttonShowHidden:SetScript("OnEnter", function() fraSQLMainAnchor.buttonShowHiddenHover:SetAlpha(1) end)
	fraSQLMainAnchor.buttonShowHidden:SetScript("OnLeave", function() fraSQLMainAnchor.buttonShowHiddenHover:SetAlpha(0) end)
	fraSQLMainAnchor.buttonShowHidden:SetScript("OnClick", function() 
		db.Frames.ShowAllQuests = not db.Frames.ShowAllQuests
		if (db.Frames.ShowAllQuests == true) then
			fraSQLMainAnchor.buttonShowHidden:SetBackdropColor(0, 0.3, 1, 1)
		else
			fraSQLMainAnchor.buttonShowHidden:SetBackdropColor(0.5, 0.5, 0.5, 1)
		end
		self:UpdateMinion()
	end)

	
	self:doAnchorBackgroundUpdate()
	self:doHiddenQuestsUpdate()
	self:ShowSorhaQuestLog()
	
	local strButtonName = ""
	local i = 1
	for i=1,intNumberOfFrames,1 do
		strButtonName = strButtonPrefix .. i
		dicButtons[strButtonPrefix .. i] = self:doCreateLooseFrame("BUTTON",strButtonName,fraSQLMainAnchor,10,10,1,"LOW",1,1)
		dicButtons[strButtonPrefix .. i].objFontString = dicButtons[strButtonPrefix .. i]:CreateFontString(nil, "OVERLAY");
		dicButtons[strButtonPrefix .. i].objFontString:SetFont(LSM:Fetch("font", db.Frames.HeaderFont), 11, db.Frames.HeaderFontOutline)
		dicButtons[strButtonPrefix .. i].objFontString:SetPoint("LEFT", dicButtons[strButtonPrefix .. i], "LEFT", 0, 0);
		dicButtons[strButtonPrefix .. i].objFontString:SetJustifyH("LEFT")
		dicButtons[strButtonPrefix .. i].objFontString:SetJustifyV("TOP")
		dicButtons[strButtonPrefix .. i].objFontString:SetText("");
		dicButtons[strButtonPrefix .. i].objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		dicButtons[strButtonPrefix .. i].objFontString:SetShadowOffset(1, -1)
		dicButtons[strButtonPrefix .. i]:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
		dicButtons[strButtonPrefix .. i]:SetBackdropColor(0, 0, 0, 0)
		dicButtons[strButtonPrefix .. i]:SetBackdropBorderColor(0, 0, 0, 0)
		
		

		strButtonName = strButtonPrefix .. "item" .. i
		dicButtons[strButtonPrefix .. i].fraItemButton = CreateFrame("BUTTON", strButtonName, dicButtons[strButtonPrefix .. i], "WatchFrameItemButtonTemplate");
		dicButtons[strButtonPrefix .. i].fraItemButton:SetScale(0.75)

		dicButtons[strButtonPrefix .. i].isHeader = nil
		dicButtons[strButtonPrefix .. i].isCollapsed = nil
		dicButtons[strButtonPrefix .. i].isUsed = false
		
		dicButtons[strButtonPrefix .. i]:RegisterForClicks("AnyUp")
		dicButtons[strButtonPrefix .. i]:SetScript("OnClick", function(self, button)
			blnWasAClick = true
			if (button == "LeftButton") then
				if (dicButtons[strButtonPrefix .. i].isHeader == 1) then
					if (dicButtons[strButtonPrefix .. i].isCollapsed == 1) then
						ExpandQuestHeader(i)
					else
						CollapseQuestHeader(i)
					end
				else
					if (IsShiftKeyDown()) then
						ChatEdit_InsertLink(GetQuestLink(i))
					else
						ShowUIPanel(QuestLogFrame);
						SelectQuestLogEntry(i);
						QuestLog_SetSelection(i);
						QuestLog_Update();
					end
				end
			else
				local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
				if not(dicButtons[strButtonPrefix .. i].isHeader == 1) then
					if not(tContains(dbChar.Quests.HiddenQuests, questID)) then
						tinsert(dbChar.Quests.HiddenQuests, questID)	
					else
						SorhaQuestLog:RemoveObject(dbChar.Quests.HiddenQuests, questID)
					end
					SorhaQuestLog:UpdateMinion()
				end
			end
		end)
		dicButtons[strButtonPrefix .. i]:SetScript("OnEnter", function(self)
			if (not(dicButtons[strButtonPrefix .. i].isHeader)) then
				SelectQuestLogEntry(i);
				local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
				if (db.Frames.MoveItemsAndTooltipsRight == true) then
					GameTooltip:SetOwner(dicButtons[strButtonPrefix .. i], "ANCHOR_RIGHT", 0, -50);
				else 
					GameTooltip:SetOwner(dicButtons[strButtonPrefix .. i], "ANCHOR_LEFT", 0, -50);
				end
				
				local strQuestDescription, strQuestObjectives = GetQuestLogQuestText();
				local objColour = GetQuestDifficultyColor(level);
				GameTooltip:SetText(questTitle, 0, 1, 0, 1);
				GameTooltip:AddLine(strQuestObjectives, 0.5, 0.5, 0.5, 1);
				
				local strQuestTag = ""
				if (isDaily) then
					strQuestTag = strQuestTag .. L["Daily "]
				end				
				if (questTag) then
					strQuestTag = strQuestTag .. questTag
					if (suggestedGroup > 0) then
						strQuestTag = strQuestTag .. " (" .. suggestedGroup .. ")"
					end
				end				
				GameTooltip:AddLine(strQuestTag, 1, 0, 0, 1);
				
				local intPartyMembers = GetNumPartyMembers();
				local blnIsOnQuest = false
				local k = 1
				if (intPartyMembers > 0) then
					GameTooltip:AddLine(L["Party members on quest:"], 0, 1, 0, 1);
					for k = 1, intPartyMembers do
						blnIsOnQuest = IsUnitOnQuest(i, "party" .. k);
						if (blnIsOnQuest == 1) then
							GameTooltip:AddLine(UnitName("party" .. k), 1, 1, 1, 1);
						end
					end				
				end
				
				GameTooltip:Show();
			end
		end)
		dicButtons[strButtonPrefix .. i]:SetScript("OnLeave", function(self) 
			GameTooltip:Hide() 
		end)

	end
end

function SorhaQuestLog:UpdateMinion()
	local fraTempHeaderFrame = fraSQLMainAnchor
	local intHeaderPosition = 0
	local intQuestOffset = 10
	local intSpacingIncrease = 0
	local intLargestWidth = 0
	local blnLargestWidthIsHeader = false
	local intHeaderOutlineOffset = 0
	local intQuestOutlineOffset = 0
	blnWasAClick = false

	--Add in slight offsets for outlined text to try stop overlap
	if (db.Frames.HeaderFontOutline == "THICKOUTLINE") then
		intHeaderOutlineOffset = 3
	elseif (db.Frames.HeaderFontOutline == "OUTLINE") then
		intHeaderOutlineOffset = 1
	end
	if (db.Frames.QuestFontOutline == "THICKOUTLINE") then
		intQuestOutlineOffset = 1
	elseif (db.Frames.QuestFontOutline == "OUTLINE") then
		intQuestOutlineOffset = 0.5
	end
	
	local numEntries, numQuests = GetNumQuestLogEntries();	
	if (db.Frames.ShowNumberOfQuests == true) then
		fraSQLMainAnchor.objFontString:SetFont(LSM:Fetch("font", db.Frames.HeaderFont), 14, db.Frames.HeaderFontOutline)
		fraSQLMainAnchor.objFontString:SetText(strHeaderColour .. numQuests .. "/25|r");
	else
		fraSQLMainAnchor.objFontString:SetText("");
	end
	fraSQLMainAnchor:SetWidth(db.Frames.MinionWidth)
	
	local i = 1
	for i=1,intNumberOfFrames,1 do
		dicButtons[strButtonPrefix .. i]:Hide()
		dicButtons[strButtonPrefix .. i].fraItemButton:Hide()
		dicButtons[strButtonPrefix .. i].objFontString:SetText("")
		dicButtons[strButtonPrefix .. i].objFontString:SetWidth(0)
		dicButtons[strButtonPrefix .. i].objFontString:SetHeight(0)
		dicButtons[strButtonPrefix .. i]:ClearAllPoints()
		dicButtons[strButtonPrefix .. i].isUsed = false
	end

	for i = 1, numEntries, 1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		dicButtons[strButtonPrefix .. i].isHeader = isHeader
		dicButtons[strButtonPrefix .. i].isCollapsed = isCollapsed
		intSpacingIncrease = 0
		
		if (questTitle == nil) then
			questTitle = L["Unknown"]
		end
		
		if (not isHeader) then
			if ((db.Frames.ShowAllQuests == true or (tContains(dbChar.Quests.HiddenQuests, questID) == nil)) or db.Frames.AllowHiddenQuests == false) then
				dicButtons[strButtonPrefix .. i]:Show()
				dicButtons[strButtonPrefix .. i].isUsed = true
				local strQuestText = self:GetQuestText(level, questTag, suggestedGroup, isDaily, questTitle, isComplete, i, questID)

				dicButtons[strButtonPrefix .. i]:SetPoint("TOP",fraSQLMainAnchor,"CENTER",intQuestOffset,intHeaderPosition)
				dicButtons[strButtonPrefix .. i].objFontString:SetFont(LSM:Fetch("font", db.Frames.QuestFont), db.Frames.QuestFontSize, db.Frames.QuestFontOutline)
				dicButtons[strButtonPrefix .. i].objFontString:SetText(strQuestText);

				dicButtons[strButtonPrefix .. i]:SetWidth(db.Frames.MinionWidth - intQuestOffset*2)
				--
				if (dicButtons[strButtonPrefix .. i].objFontString:GetWidth()  > intLargestWidth) then
					intLargestWidth = dicButtons[strButtonPrefix .. i].objFontString:GetWidth()
					blnLargestWidthIsHeader = false
				end
				--
				dicButtons[strButtonPrefix .. i].objFontString:SetWidth(db.Frames.MinionWidth - intQuestOffset*2)
				intSpacingIncrease = dicButtons[strButtonPrefix .. i].objFontString:GetHeight() + intQuestOutlineOffset
			
				if (db.Frames.ShowItemButtons == true) then
					local link, item, charges = GetQuestLogSpecialItemInfo(i);
					if (item) then
						dicButtons[strButtonPrefix .. i].fraItemButton:ClearAllPoints();
						if (db.Frames.MoveItemsAndTooltipsRight == true) then
							dicButtons[strButtonPrefix .. i].fraItemButton:SetPoint("TOPLEFT", dicButtons[strButtonPrefix .. i], "TOPRIGHT", 7, 0)
						else
							dicButtons[strButtonPrefix .. i].fraItemButton:SetPoint("TOPRIGHT", dicButtons[strButtonPrefix .. i], "TOPLEFT", -12, 0)
						end
						dicButtons[strButtonPrefix .. i].fraItemButton:Show()
						dicButtons[strButtonPrefix .. i].fraItemButton:SetID(i)
						SetItemButtonTexture(dicButtons[strButtonPrefix .. i].fraItemButton, item);
						SetItemButtonCount(dicButtons[strButtonPrefix .. i].fraItemButton, charges);
						WatchFrameItem_UpdateCooldown(dicButtons[strButtonPrefix .. i].fraItemButton);
						
						-- If a button is heigher then its quest then expand the quest frame to stop overlapping buttons
						local intButtonHeight = dicButtons[strButtonPrefix .. i].fraItemButton:GetHeight() * dicButtons[strButtonPrefix .. i].fraItemButton:GetScale()
						if (intButtonHeight > intSpacingIncrease) then
							intSpacingIncrease = intButtonHeight
							dicButtons[strButtonPrefix .. i].objFontString:SetHeight(intButtonHeight)
						end
					end
				end
				dicButtons[strButtonPrefix .. i]:SetHeight(intSpacingIncrease)
			end
		else
			dicButtons[strButtonPrefix .. i]:Show()
			dicButtons[strButtonPrefix .. i].isUsed = true
			local strPrefix = strHeaderColour .. "- "
			if (isCollapsed == 1) then
				strPrefix = strHeaderColour .. "+ "
			end	

			dicButtons[strButtonPrefix .. i]:SetPoint("TOP",fraSQLMainAnchor,"CENTER",0,intHeaderPosition)
			dicButtons[strButtonPrefix .. i].objFontString:SetFont(LSM:Fetch("font", db.Frames.HeaderFont), db.Frames.HeaderFontSize, db.Frames.HeaderFontOutline)
			dicButtons[strButtonPrefix .. i].objFontString:SetText(strPrefix .. questTitle .. "|r");
			
			dicButtons[strButtonPrefix .. i]:SetWidth(db.Frames.MinionWidth)
			--
			if (dicButtons[strButtonPrefix .. i].objFontString:GetWidth() > intLargestWidth) then
				intLargestWidth = dicButtons[strButtonPrefix .. i].objFontString:GetWidth()
				blnLargestWidthIsHeader = true
			end
			--
			dicButtons[strButtonPrefix .. i].objFontString:SetWidth(db.Frames.MinionWidth)
			
			intSpacingIncrease = dicButtons[strButtonPrefix .. i].objFontString:GetHeight() + intHeaderOutlineOffset
			dicButtons[strButtonPrefix .. i]:SetHeight(intSpacingIncrease)
		end
		intHeaderPosition = intHeaderPosition - intSpacingIncrease
    end
	if (db.Frames.CollapseToLeft == true) then
		local intNonQuestSize = 0
		local intQuestSize = 0
		if (blnLargestWidthIsHeader == true) then
			intNonQuestSize = intLargestWidth
			intQuestSize = intLargestWidth - (intQuestOffset * 2)
		else
			intNonQuestSize = intLargestWidth + (intQuestOffset * 2)
			intQuestSize = intLargestWidth	
		end
	
		if (intNonQuestSize < db.Frames.MinionWidth) then
			fraSQLMainAnchor:SetWidth(intNonQuestSize)
			for i = 1, numEntries, 1 do
				if (dicButtons[strButtonPrefix .. i].isUsed == true) then
					if (dicButtons[strButtonPrefix .. i].isHeader == 1) then
						dicButtons[strButtonPrefix .. i].objFontString:SetWidth(intNonQuestSize)
						dicButtons[strButtonPrefix .. i]:SetWidth(intNonQuestSize)
					else
						dicButtons[strButtonPrefix .. i].objFontString:SetWidth(intQuestSize)
						dicButtons[strButtonPrefix .. i]:SetWidth(intQuestSize)
					end
				end
			end
		end
	end
	blnIsUpdating = false
end

function SorhaQuestLog:GetQuestText(level, questTag, suggestedGroup, isDaily, questTitle, isComplete, i, questID)
	local strTitleColor = ""
	if (db.Frames.UseQuestLevelColouring == true) then
		local objColour = GetQuestDifficultyColor(level);	
		strTitleColor = format("|c%02X%02X%02X%02X", 255, objColour.r * 255, objColour.g * 255, objColour.b * 255);
	else
		strTitleColor = strQuestTitleColour
	end
	local strReturnText = strTitleColor .. "[" .. level
	
	SelectQuestLogEntry(i);
	local numQuestLogLeaderBoards = GetNumQuestLeaderBoards(i)

	if (questTag == LFG_TYPE_DUNGEON) then
		strReturnText = strReturnText .. "d"
	end		
	if (questTag == RAID) then
		strReturnText = strReturnText .. "r"
	end		
	if (questTag == PVP) then
		strReturnText = strReturnText .. "p"
	end		
	if (suggestedGroup > 0) then
		strReturnText = strReturnText .. "g" .. suggestedGroup
	end
	if (isDaily) then
		strReturnText = strReturnText .. "*"
	end
	
	strReturnText = strReturnText .. "] " .. questTitle .. "|r"
	if (isComplete and isComplete < 0) then 
		strReturnText = strReturnText .. L[" (Failed)"]
	elseif (isComplete and isComplete > 0) then
		strReturnText = strReturnText .. L[" (Done)"]
	elseif (numQuestLogLeaderBoards == 0) then
		strReturnText = strReturnText .. L[" (goto)"]
	end
	
	if ((tContains(dbChar.Quests.HiddenQuests, questID) == 1) and db.Frames.AllowHiddenQuests == true) then
		strReturnText = strReturnText .. strHeaderColour .. L[" (Hidden)"] .. "|r"
	end 
	
	strReturnText = strReturnText .. "\n"

	local k = 1
	if (numQuestLogLeaderBoards > 0) then --If has objectives then for each objective print, else print quest 'objective' description
		for k=1,numQuestLogLeaderBoards,1 do
			local leaderboardTxt, itemType, isDone = GetQuestLogLeaderBoard(k,i);
		
			if (leaderboardTxt ~= nil) then --For some reason some quests have hidden objectives that are blank o_O
				if ((isDone == nil)) then
					if (db.Frames.UseProgressiveColouring == true) then
						local y, z, strObjDesc, intNumGot, intNumNeeded = string.find(leaderboardTxt, "(.*):%s*([%d]+)%s*/%s*([%d]+)");
						local strOutput = ""
						local strColourOutput = "|cffff0000"
						
						if (strObjDesc == nil) then
							strObjDesc = ""
						end						
						if (intNumGot == nil) then
							intNumGot = 0
						end
						if (intNumNeeded == nil) then
							intNumNeeded = 1
						end
						
						if (itemType == "event") then
							strOutput = leaderboardTxt
						elseif (itemType == nil) then
							strOutput = leaderboardTxt
						else
							strColourOutput = self:GetQuestObjectiveColour(intNumGot, intNumNeeded)
							strOutput = strObjDesc .. ": " .. strColourOutput .. intNumGot .. "/" .. intNumNeeded .. "|r"
						end
						strReturnText = strReturnText .. strObjectiveColour .. " - " .. strOutput .. "|r\n"
					else
						strReturnText = strReturnText .. strObjectiveColour .. " - " .. leaderboardTxt .. "|r\n"
					end

				else
					if (db.Frames.HideCompletedObjectives == false) then
						strReturnText = strReturnText .. strDoneObjectiveColour .. " - " .. leaderboardTxt .. "|r\n"
					end
				end	
			end
		end
	else 
		if (db.Frames.ShowDescWhenNoObjectives == true) then
			local strQuestDescription, strQuestObjectives = GetQuestLogQuestText();
			if (not(isComplete and isComplete > 0)) then
				strReturnText = strReturnText .. strObjectiveDescriptionColour .. " - " .. strQuestObjectives .. "|r\n"
			else
				if (db.Frames.HideCompletedObjectives == false) then
					strReturnText = strReturnText .. strObjectiveDescriptionColour .. " - " .. strQuestObjectives .. "|r\n"
				end
			end	
		end
	end
	strReturnText = strtrim(strReturnText)
	return strReturnText
end

function SorhaQuestLog:GetQuestObjectiveColour(intNumGot, intNumNeeded)
	local dblPercent = intNumGot/intNumNeeded
	if (dblPercent < 0.25) then
		return strObjective00PlusColour
	elseif (dblPercent >= 0.25 and dblPercent < 0.50) then
		return strObjective25PlusColour
	elseif (dblPercent >= 0.50 and dblPercent < 0.75) then
		return strObjective50PlusColour
	else
		return strObjective75PlusColour
	end
end

function SorhaQuestLog:doHandleZoneChange()
	local blnNewZone = not(strZone == GetRealZoneText())
	strZone = GetRealZoneText()
	strSubZone = GetSubZoneText()
	blnIgnoreUpdateEvents = true
	
	if (db.Frames.CollapseOnLeave == true or db.Frames.ExpandOnEnter == true) then
		local numEntries, numQuests = GetNumQuestLogEntries();	
		local i = 1
		for i = numEntries, 1, -1 do
			local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
			if (isHeader) then
				if (questTitle == strZone or questTitle == strSubZone) then
					if (db.Frames.ExpandOnEnter == true and isCollapsed) then
						ExpandQuestHeader(i)
					end
				else		
					if (db.Frames.CollapseOnLeave == true and not(isCollapsed) and blnNewZone) then
						CollapseQuestHeader(i)
					end				
				end
			end
		end
	end

	blnIgnoreUpdateEvents = false
	if (blnInitialized == true) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:SetupOptions() --Creates options. Registers chat commands
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SorhaQuestLog", getOptions())
	
	AceConfigDialog:SetDefaultSize("SorhaQuestLog", 680,525)
	self:RegisterChatCommand("sql", "ChatCommand")
	self:RegisterChatCommand("SorhaQuestLog", "ChatCommand")
end

function SorhaQuestLog:ChatCommand(input) --Handles chat commands
	if not input or input:trim() == "" then
		LibStub("AceConfigDialog-3.0"):Open("SorhaQuestLog")
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SorhaQuestLog, "SorhaQuestLog", "SorhaQuestLog", input)
	end
end

function SorhaQuestLog:doColourChange() --Changes the addons colours on the fly as changed
	self:SetupColours()
	self:UpdateMinion()
end

function SorhaQuestLog:SetupColours()
	strHeaderColour = format("|c%02X%02X%02X%02X", 255, db.Frames.HeaderColour.r * 255, db.Frames.HeaderColour.g * 255, db.Frames.HeaderColour.b * 255);
	strQuestTitleColour = format("|c%02X%02X%02X%02X", 255, db.Frames.QuestTitleColour.r * 255, db.Frames.QuestTitleColour.g * 255, db.Frames.QuestTitleColour.b * 255);
	strObjectiveColour = format("|c%02X%02X%02X%02X", 255, db.Frames.ObjectiveColour.r * 255, db.Frames.ObjectiveColour.g * 255, db.Frames.ObjectiveColour.b * 255);
	strObjective00PlusColour = format("|c%02X%02X%02X%02X", 255, db.Frames.Objective00PlusColour.r * 255, db.Frames.Objective00PlusColour.g * 255, db.Frames.Objective00PlusColour.b * 255);
	strObjective25PlusColour = format("|c%02X%02X%02X%02X", 255, db.Frames.Objective25PlusColour.r * 255, db.Frames.Objective25PlusColour.g * 255, db.Frames.Objective25PlusColour.b * 255);
	strObjective50PlusColour = format("|c%02X%02X%02X%02X", 255, db.Frames.Objective50PlusColour.r * 255, db.Frames.Objective50PlusColour.g * 255, db.Frames.Objective50PlusColour.b * 255);
	strObjective75PlusColour = format("|c%02X%02X%02X%02X", 255, db.Frames.Objective75PlusColour.r * 255, db.Frames.Objective75PlusColour.g * 255, db.Frames.Objective75PlusColour.b * 255);
	strDoneObjectiveColour = format("|c%02X%02X%02X%02X", 255, db.Frames.ObjectiveDoneColour.r * 255, db.Frames.ObjectiveDoneColour.g * 255, db.Frames.ObjectiveDoneColour.b * 255);
	strObjectiveDescriptionColour = format("|c%02X%02X%02X%02X", 255, db.Frames.ObjectiveDescColour.r * 255, db.Frames.ObjectiveDescColour.g * 255, db.Frames.ObjectiveDescColour.b * 255);
end

function SorhaQuestLog:CleanupHiddenQuestList() --Clean out completed/abandoned quests from hidden quests table
	local numEntries, numQuests = GetNumQuestLogEntries();	
	local i = 1
	local haveQuests = {}
	for i = 1, numEntries, 1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if (not isHeader) then
			tinsert(haveQuests, questID)
		end
	end
	
	local tempTable = {}
	for key, qID in pairs(haveQuests) do
		if (tContains(dbChar.Quests.HiddenQuests, qID) == 1) then
			tinsert(tempTable, qID)
		end
	end
	dbChar.Quests.HiddenQuests = tempTable
end

function SorhaQuestLog:RemoveObject(objTable, objFind)
	for k, v in pairs(objTable) do
		if (v == objFind) then
			tremove(objTable,k)
			break
		end
	end
end

function SorhaQuestLog:doAnchorBackgroundUpdate()
	if db.Frames.MainWindowLocked == false then
		fraSQLMainAnchor:SetBackdropColor(0, 1, 0, 0.5)
		fraSQLMainAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
		fraSQLMainAnchor:EnableMouse(true)
	else
		fraSQLMainAnchor:SetBackdropColor(0, 1, 0, 0)
		fraSQLMainAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
		fraSQLMainAnchor:EnableMouse(false)
	end
end

function SorhaQuestLog:doHiddenQuestsUpdate()
	if (db.Frames.AllowHiddenQuests == true) then
		fraSQLMainAnchor.buttonShowHidden:Show()	
		fraSQLMainAnchor.buttonShowHiddenHover:Show()
		fraSQLMainAnchor.objFontString:ClearAllPoints()
		fraSQLMainAnchor.objFontString:SetPoint("TOPLEFT", fraSQLMainAnchor, "TOPLEFT", 16, -5);
		if (db.Frames.ShowAllQuests == true) then
			fraSQLMainAnchor.buttonShowHidden:SetBackdropColor(0, 0.3, 1, 1)
		else
			fraSQLMainAnchor.buttonShowHidden:SetBackdropColor(0.5, 0.5, 0.5, 1)
		end
	else
		fraSQLMainAnchor.buttonShowHidden:Hide()	
		fraSQLMainAnchor.buttonShowHiddenHover:Hide()
		fraSQLMainAnchor.objFontString:ClearAllPoints()
		fraSQLMainAnchor.objFontString:SetPoint("TOPLEFT", fraSQLMainAnchor, "TOPLEFT", 0, -5);
	end
	if (blnInitialized == true) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:doCreateLooseFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,intAlpha,finherit)
	local f = CreateFrame(strType,strName,fraParent,finherit)
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)	
	f:SetAlpha(intAlpha)
	return f  
end 

function SorhaQuestLog:doCreateFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY,intAlpha,finherit)
	local f = CreateFrame(strType,strName,fraParent,finherit)
	
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)
	f:SetPoint(strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY)
	f:SetAlpha(intAlpha)
	return f  
end 

function SorhaQuestLog:ShowSorhaQuestLog()
	db.Frames.HideSorhaQuestLog = false
	fraSQLMainAnchor:Show()
	if (blnInitialized == true) then
		self:UpdateMinion()
	end
end

function SorhaQuestLog:HideSorhaQuestLog()
	db.Frames.HideSorhaQuestLog = true
	fraSQLMainAnchor:Hide()
end

function SorhaQuestLog:ToggleSorhaQuestLog()
	if (db.Frames.HideSorhaQuestLog == true) then
		self:ShowSorhaQuestLog()
	else
		self:HideSorhaQuestLog()
	end
end